      SUBROUTINE ZERF(ZR,ZI,KODE,ZANSR,ZANSI,IERR)
C 
C     WRITTEN BY D.E. AMOS, FEBRUARY, 2002
C 
C     REFERENCE 
C         NBS HANDBOOK OF MATHEMATICAL FUNCTIONS, AMS 55, BY
C         M. ABRAMOWITZ AND I.A. STEGUN, DECEMBER,1955, CHAPTER 7. 
C 
C     ABSTRACT
C                      ***A DOUBLE PRECISION ***
C         A Z PREFIX BEFORE A FUNCTION OR SUBROUTINE INDICATES A
C         DOUBLE PRECISON COMPLEX ROUTINE. DOUBLE PRECISION COMPLEX
C         NUMBERS ARE CARRIED AS ORDERED PAIRS, I.E. Z=(ZR,ZI).
C
C         ZERF COMPUTES THE ERROR FUNCTION, ERF(Z), OR THE SCALED 
C         COERROR FUNCTION, EXP(Z**2)*ERFC(Z), FOR Z=CMPLX(X,Y).
C         IF X.LT.0.0, Z IS REPLACED BY -Z AND ALL COMPUTATION IS
C         DONE IN THE RIGHT HALF PLANE, EXCEPT FOR Z INSIDE THE CIRCLE
C         ZABS(Z).LT.RLB. ERF(-Z)=-ERF(Z) AND ERFC(-Z)=2.0-ERFC(Z).
C
C         THE REGIONS FOR COMPUTATION ARE DIVIDED AS FOLLOWS:
C
C           (1) ZABS(Z).LE.RLB. - POWER SERIES, NBS HANDBOOK, P. 297
C           (2) ZABS(Z).GT.RLB AND 0.LE.X.LE.RLB AND CABS(Z).LT.6 - 
C               SERIES, NBS HANDBOOK, P. 299 
C           (3) X.GT.RLB AND ZABS(Z).LT.6.0 - CONTINUED FRACTION, NBS
C               HANDBOOK, P. 298
C           (4) ZABS(Z).GE.6.- ASYMPTOTIC EXPANSION, NBS HANDBOOK, P.298
C
C               RLB IS SET IN A DATA STATEMENT SO THAT THE ERRORS
C               ON EACH SIDE OF RLB ARE ABOUT THE SAME (RLB=1.50D0).
C
C         THE SERIES IN (1) IS A CONFLUENT HYPERGEOMETRIC POWER SERIES
C         WHILE THE SERIES IN (3) IS TAKEN FROM MATH COMP,5, 67-70(1951)
C 
C         THE WORKING TOLERANCE IS MAX(UNIT ROUNDOFF, 1.0E-14)
C
C         SEVERAL COMMON FUNCTIONS ARE SPECIAL CASES -- 
C 
C         DAWSON-S INTEGRAL 
C 
C               F(Y)=-I*SQRT(PI)*EXP(-Y*Y)*ERF(I*Y)/2.
C 
C         FRESNEL INTEGRALS 
C 
C               C(Z)+I*S(Z)=(0.5+0.5*I)*ERF(SQRT(PI)*(1.-I)*Z/2.) 
C 
C         PLASMA DISPERSION FUNCTION
C 
C               W(Z)= EXP((-I*Z)**2)*ERFC(-I*Z) 
C                   = EXP( - Z*Z   )*ERFC(-I*Z) 
C 
C     DESCRIPTION OF ARGUMENTS
C         INPUT 
C           ZR     - REAL PART OF COMPLEX ARGUMENT Z
C           ZI     - IMAGINARY PART OF COMPLEX ARGUMENT Z
C           KODE   - A SELECTION PARAMETER
C                    KODE=1 RETURNS VALUES FOR ERF(Z) 
C                    KODE=2 RETURNS VALUES FOR EXP(Z**2)*ERFC(Z) WHERE
C                           ERFC(Z)=1.0-ERF(Z) 
C 
C         OUTPUT
C           ZANSR  - REAL PART OF A COMPLEX NUMBER FOR ERF(Z) OR
C                    EXP(Z**2)*ERFC(Z) DEPENDING ON KODE
C           ZANSI  - IMAGINARY PART OF A COMPLEX NUMBER FOR ERF(Z) OR
C                    EXP(Z**2)*ERFC(Z) DEPENDING ON KODE
C 
C     ERROR CONDITIONS
C         IERR=0 NORMAL RETURN, COMPUTATION COMPLETED.
C         IERR=1 KODE NOT 1 OR 2, NO RESULT,
C                ZANS=(0.0D0,0.0D0) RETURNED.
C         IERR=2 OVERFLOW, NO RESULT, ZANS=(0.0D0,0.0D0) RETURNED.
C
C     USES SUBROUTINES I1MACH,R1MACH,ZABS,ZMLT,ZDIV
C-----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)      
C     FNORM=2/SQRT(PI), GNORM=1/SQRT(PI)
      DATA FNORM,GNORM /1.12837916709551D0,  0.564189583547756D0/
C     EH=DEXP(-0.50D0), EF=DEXP(-0.25D0)
      DATA EH,EF       /0.606530659712633D0, 0.778800783071405D0/
      DATA PI          /3.14159265358979D0/
      DATA RLB         /1.50D0/
      IERR=0
      IF(KODE.LT.1.OR.KODE.GT.2) THEN
        IERR=1
        ZANSR=0.0D0
        ZANSI=0.0D0
        RETURN
      ENDIF
      TOL=MAX(D1MACH(4),1.0D-15)
      KK=MIN(IABS(I1MACH(15)),IABS(I1MACH(16)))
      ELIM=2.303D0*DBLE((FLOAT(KK))*D1MACH(5)-3.0D0)
C-----------------------------------------------------------------------
C     SERIES CEXP(Z*Z)*ERF(Z) FOR ZABS(Z).LT.RLB
C-----------------------------------------------------------------------
      AZ=ZABS(ZR,ZI)
      IF(AZ.LT.DSQRT(10.0D0*D1MACH(1))) THEN
        CZR=0.0D0
        CZI=0.0D0
      ELSE
        CALL ZMLT(ZR,ZI,ZR,ZI,CZR,CZI) 
      ENDIF
      IF(AZ.GE.RLB) GO TO 20
      SUMR=1.0D0
      SUMI=0.0D0
      TERMR=SUMR
      TERMI=SUMI
      AK=1.5D0
      RTOL=TOL/10.0D0
      DO 5 I=1,100
        CALL ZMLT(TERMR,TERMI,CZR,CZI,TEMPR,TEMPI)
        TERMR=TEMPR/AK
        TERMI=TEMPI/AK
        SUMR=SUMR+TERMR
        SUMI=SUMI+TERMI
        AT=ZABS(TERMR,TERMI)
        ASUM=ZABS(SUMR,SUMI)
        IF(AT.LE.RTOL*ASUM) GOTO 10
        AK=AK+1.0D0
    5 CONTINUE
   10 CONTINUE
      CZR=-CZR
      CZI=-CZI
      CALL ZMLT(SUMR,SUMI,ZR,ZI,ANSR,ANSI)
      SUMR=FNORM*ANSR
      SUMI=FNORM*ANSI
      TEMPR=DEXP(CZR)
      CZR=TEMPR*DCOS(CZI)
      CZI=TEMPR*DSIN(CZI)
      IF(KODE.EQ.1) THEN
        CALL ZMLT(SUMR,SUMI,CZR,CZI,ZANSR,ZANSI)
      ELSE
        TEMPR=1.0D0
        TEMPI=0.0D0
        CALL ZDIV(TEMPR,TEMPI,CZR,CZI,CZR,CZI)
        ZANSR=CZR-SUMR
        ZANSI=CZI-SUMI
      ENDIF
      RETURN
   20 CONTINUE
C-----------------------------------------------------------------------
C     COMPUTE IN THE RIGHT HALF PLANE FOR (2),(3) & (4) IN THE PROLOGUE
C-----------------------------------------------------------------------
      ZPR=ZR
      ZPI=ZI
      X=ZR
      Y=ZI
      IF(X.LT.0.0D0) THEN
        ZPR=-ZR
        ZPI=-ZI
      ENDIF
      CALL ZMLT(ZPR,ZPI,ZPR,ZPI,CZR,CZI)
      IF(AZ.GT.6.0D0) GOTO 200
      XP=ZPR
      YP=ZPI
      IF(XP.LE.RLB) GOTO 150
C-----------------------------------------------------------------------
C     CONTINUED FRACTION FOR ERFC(Z), X.GT.RLB AND CABS(Z).LE.6.0. 
C     N=INT(75.0/AZ+5.0) FOR TOL=MAX(UNIT ROUND OFF,1.E-15)
C-----------------------------------------------------------------------
      N=INT(SNGL((75.0D0/AZ+5.0D0)))
      FN=DBLE(FLOAT(N))
      TERMR=CZR
      TERMI=CZI
      DO 30 I=1,N 
        FNH=FN-0.5D0
        TEMPR=FN+TERMR
        TEMPI=TERMI
        ANSR=FNH*TERMR
        ANSI=FNH*TERMI
        CALL ZDIV(ANSR,ANSI,TEMPR,TEMPI,TEMPR,TEMPI)
        TERMR=CZR+TEMPR
        TERMI=CZI+TEMPI
        FN=FN-1.0D0
   30 CONTINUE
      IF(KODE.EQ.2) GO TO 35
      CZR=-CZR
      CZI=-CZI
      IF(DABS(CZR).GT.ELIM) THEN
        IF(CZR.GT.0.0D0) THEN
          IERR=2
          ZANSR=0.0D0
          ZANSI=0.0D0
          RETURN
        ELSE
          RER=0.0D0
        ENDIF
      ELSE
        RER=DEXP(CZR)
      ENDIF
      CZR=RER*DCOS(CZI)
      CZI=RER*DSIN(CZI)
      TEMPR=GNORM*ZPR
      TEMPI=GNORM*ZPI
      CALL ZDIV(TEMPR,TEMPI,TERMR,TERMI,SUMR,SUMI)
      GOTO 203
   35 CONTINUE
      TEMPR=GNORM*ZPR
      TEMPI=GNORM*ZPI
      CALL ZDIV(TEMPR,TEMPI,TERMR,TERMI,ZANSR,ZANSI)
      IF(X.GE.0.0D0) RETURN
      IF(DABS(CZR).GT.ELIM) THEN
        IF(CZR.GT.0.0D0) THEN
          IERR=2
          ZANSR=0.0D0
          ZANSI=0.0D0
          RETURN
        ELSE
          RER=0.0D0
        ENDIF
      ELSE
        RER=DEXP(CZR)
      ENDIF
      CEIR=DCOS(CZI)
      CEII=DSIN(CZI)
      CZR=RER*CEIR
      CZI=RER*CEII
      ZANSR=CZR+CZR-ZANSR
      ZANSI=CZI+CZI-ZANSI
      RETURN 
  150 CONTINUE
C-----------------------------------------------------------------------
C    SERIES (2) FOR ZABS(Z).GE.RLB AND 0.0.LE.XP.LE.RLB AND ZABS(Z).LT.6
C-----------------------------------------------------------------------
      S1=0.0D0
      S2=0.0D0
      IF(DABS(XP).LT.SQRT(D1MACH(1))) THEN
        X2=0.0D0
      ELSE
        X2=XP*XP
      ENDIF
      AY=DABS(YP)
      IF(AY.NE.0.0D0) THEN
C       TOL IS DECREASED BY 100 TO BE SLIGHTLY CONSERVATIVE IN DECIDING
C       WHEN EXP(-2*N*Y) IS SMALL COMPARED TO 1.0D0
        ETOL=DLOG(TOL/100.0D0) 
        FKK=MIN(100.0D0,-0.5D0*ETOL/AY+2.0D0)
        KK=INT(FKK)
        FKK=MIN(100.0D0,ELIM/AY+2.0D0)
        KKP=INT(FKK)
      ELSE
        SXY=0.0D0
        CXY=1.0D0
        EX=DEXP(-X2)
        GOTO 96
      ENDIF 
      FX2=4.0D0*X2
      TX=XP+XP
      XY=XP*AY
      TXY=XY+XY 
      SXYH=DSIN(XY)
      SXY=DSIN(TXY)
      CXY=DCOS(TXY)
      FN=1.0D0
      FNH=0.5D0
      EY=DEXP(AY)
      REY=1.0D0/EY
      REY2=REY*REY
      REY2N=REY2
      TXM=TX*REY
      EHN=EH
      UN=EF*EY
      VN=1.0D0
      RTOL=TOL/10.0D0
      DO 90 I=1,100
        CSH=1.0D0+REY2N
        TM=XP*CSH
        SSH=1.0D0-REY2N
        TMP=FNH*SSH
        RN=TXM-TM*CXY+TMP*SXY
        AIN=TM*SXY+TMP*CXY
        CF=UN/(VN+FX2)
        RN=CF*RN
        AIN=CF*AIN
        S1=S1+RN
        S2=S2+AIN
        AT=DABS(RN)+DABS(AIN)
        AS=DABS(S1)+DABS(S2)
        IF(AT.LE.RTOL*AS) GO TO 95
        UN=UN*EHN*EF*EY
        EHN=EHN*EH
        IF (I.GE.KK) THEN
          REY2N=0.0D0
        ELSE
          REY2N=REY2N*REY2
        ENDIF
        IF(I.GE.KKP) THEN
          TXM=0.0D0
        ELSE
          TXM=TXM*REY
        ENDIF
        VN=VN+FN+FN+1.0D0
        FNH=FNH+0.5D0
        FN=FN+1.0D0
   90 CONTINUE
   95 S1=S1+S1
      S2=S2+S2
      IF(X.NE.0.0D0) THEN
        S1=S1+SXYH*SXYH/XP
        S2=S2+SXY/TX
      ELSE
        S2=S2+AY
      ENDIF
C-----------------------------------------------------------------------
C     POWER SERIES FOR DEXP(XP*XP)*DERF(XP), 0.LE.XP.LE.RLB
C-----------------------------------------------------------------------
   96 CONTINUE   
      W=1.0D0
      AK=1.5D0
      TM=1.0D0
      RTOL=TOL/10.0D0
      DO 98 I=1,100
        TM=TM*X2/AK
        W=W+TM
        IF(DABS(TM).LE.DABS(W)*RTOL) GO TO 99
        AK=AK+1.0D0
   98 CONTINUE
   99 CONTINUE
      EX=DEXP(-X2)
      W=W*XP*FNORM
      IF(KODE.EQ.2) GO TO 100 
      CF=EX/PI
      S1=S1*CF+W*EX
      S2=S2*CF
      IF(YP.LT.0.0D0) S2=-S2
      ZANSR=S1
      ZANSI=S2
      IF(X.LT.0.0D0) THEN
        ZANSR=-ZANSR
        ZANSI=-ZANSI
      ENDIF 
      RETURN
  100 CONTINUE
      IF((X.NE.0.0D0).AND.(Y.NE.0.0D0)) THEN
        SXY=SXY*DSIGN(1.0D0,Y)*DSIGN(1.0D0,X)
      ENDIF
      RCZR=CXY
      RCZI=SXY
      Y2=YP*YP
      EY=DEXP(-Y2)
      RER=EY/EX
      CZR=RER*RCZR
      CZI=RER*RCZI
      S1=EY*(W+S1/PI)
      S2=EY*S2/PI
      IF(YP.LT.0.0D0) S2=-S2
      IF(X.LT.0.0D0) THEN
        S1=-S1
        S2=-S2
      ENDIF
      CALL ZMLT(RCZR,RCZI,S1,S2,TEMPR,TEMPI)
      ZANSR=CZR-TEMPR
      ZANSI=CZI-TEMPI
      RETURN
  200 CONTINUE
C-----------------------------------------------------------------------
C     ASYMPTOTIC EXPANSION FOR CABS(Z).GT.6 
C-----------------------------------------------------------------------
      TEMPR=0.5D0
      TEMPI=0.0D0
      CALL ZDIV(TEMPR,TEMPI,CZR,CZI,RCZR,RCZI)
      SUMR=1.0D0
      SUMI=0.0D0
      TERMR=SUMR
      TERMI=SUMI
      AK=1.0D0
      DO 201 I=1,35
        CALL ZMLT(TERMR,TERMI,RCZR,RCZI,TEMPR,TEMPI)
        TERMR=-AK*TEMPR
        TERMI=-AK*TEMPI
        SUMR=SUMR+TERMR
        SUMI=SUMI+TERMI
        AT=ZABS(TERMR,TERMI)
        AS=ZABS(SUMR,SUMI)
        IF(AT.LE.TOL*AS) GO TO 202
        AK=AK+2.0D0
  201 CONTINUE
      PRINT *,"DROP THRU ASYMPTOTIC EXPANSION LOOP-- AN ERROR FOR"
      PRINT *,"Z = ",ZR,ZI, " AND KODE = ",KODE
  202 CONTINUE
      ANSR=SUMR*GNORM
      ANSI=SUMI*GNORM
      CALL ZDIV(ANSR,ANSI,ZPR,ZPI,SUMR,SUMI)
      IF(KODE.EQ.2) GO TO 206
      CZR=-CZR
      CZI=-CZI
      IF(DABS(CZR).GT.ELIM) THEN
        IF(CZR.GT.0.0D0) THEN
          IERR=2
          ZANSR=0.0D0
          ZANSI=0.0D0
          RETURN
        ELSE
          RER=0.0D0
        ENDIF
      ELSE
        RER=DEXP(CZR)
      ENDIF
      CZR=RER*DCOS(CZI)
      CZI=RER*DSIN(CZI)
C-----------------------------------------------------------------------
C     ERF(Z)=1.0-ERFC(Z), 
C     FOR ABS(Z) NEAR 6 ERFC(Z)=O(EXP(-Z*Z)=EXP(Y*Y-X*X)) AND ERFC IS
C     SUBTRACTED FROM 1.0 ONLY IF ERFC AND 1.0-ERFC HAVE DIFFERENT
C     MAGNITUDES. THIS PRESERVES THE ODDNESS OF ERF WHEN Z IS CLOSE TO
C     I*Y WHERE ERF(Z) IS PURELY IMAGINARY. (Y*Y-X*X IS RELATIVELY LARGE
C     FOR Y>X, I.E. EXP(36) IS O(10**(16)) AND 1.0-EXP(36)=EXP(36) TO
C     MACHINE PRECISION.) ABS((1-U)*(1-U)+V*V)=U*U+V*V+1-2*U AND 1-2U IS
C     THE CHANGE IN MAGNITUDE SQUARED. THEREFORE ABS(1-2*U)/(U*U+V*V)
C     MUST BE LESS THAN UNIT ROUNDOFF TO SEE NO CHANGE IN MAGNITUDE
C     WHERE U=REAL(ERFC(Z)) AND V=AIMAG(ERFC(Z)).
C-----------------------------------------------------------------------
  203 CONTINUE
      CALL ZMLT(-CZR,-CZI,SUMR,SUMI,TERMR,TERMI)
      ATERM=ZABS(TERMR,TERMI)
      DEL=DABS(1.0D0-TERMR-TERMR)
      RTOL=DSQRT(TOL)
      IF(SQRT(DEL).LT.ATERM*RTOL) THEN
        ZANSR=TERMR
        ZANSI=TERMI
      ELSE
        ZANSR=1.0D0+TERMR
        ZANSI=TERMI
      ENDIF
      IF(X.LT.0.0D0) THEN
        ZANSR=-ZANSR
        ZANSI=-ZANSI
      ENDIF
      RETURN
  206 CONTINUE
C-----------------------------------------------------------------------
C     CEXP((-Z)**2)*ERFC(-Z)=2.0*CEXP(Z*Z)-CEXP(Z*Z)*ERFC(Z)
C-----------------------------------------------------------------------
      IF(X.LT.0.0D0)THEN
        IF(DABS(CZR).GT.ELIM) THEN
          IF(CZR.GT.0.0D0) THEN
            IERR=2
            ZANSR=0.0D0
            ZANSI=0.0D0
            RETURN
          ELSE
            RER=0.0D0
          ENDIF
        ELSE
          RER=DEXP(CZR)
        ENDIF
        CEIR=DCOS(CZI)
        CEII=DSIN(CZI)
        CZR=RER*CEIR
        CZI=RER*CEII
        SUMR=CZR+CZR-SUMR
        SUMI=CZI+CZI-SUMI
      ENDIF
      ZANSR=SUMR
      ZANSI=SUMI
      RETURN
      END
      SUBROUTINE ZMLT(AR, AI, BR, BI, CR, CI)
C***BEGIN PROLOGUE  ZMLT
C
C     DOUBLE PRECISION COMPLEX MULTIPLY, C=A*B.
C
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  ZMLT
      DOUBLE PRECISION AR, AI, BR, BI, CR, CI, CA, CB
      CA = AR*BR - AI*BI
      CB = AR*BI + AI*BR
      CR = CA
      CI = CB
      RETURN
      END
      SUBROUTINE ZDIV(AR, AI, BR, BI, CR, CI)
C***BEGIN PROLOGUE  ZDIV
C
C     DOUBLE PRECISION COMPLEX DIVIDE C=A/B.
C
C***ROUTINES CALLED  ZABS
C***END PROLOGUE  ZDIV
      DOUBLE PRECISION AR, AI, BR, BI, CR, CI, BM, CA, CB, CC, CD
      DOUBLE PRECISION ZABS
      BM = 1.0D0/ZABS(BR,BI)
      CC = BR*BM
      CD = BI*BM
      CA = (AR*CC+AI*CD)*BM
      CB = (AI*CC-AR*CD)*BM
      CR = CA
      CI = CB
      RETURN
      END
      SUBROUTINE ZSQRT(AR, AI, BR, BI)
C***BEGIN PROLOGUE  ZSQRT
C
C     DOUBLE PRECISION COMPLEX SQUARE ROOT, B=CSQRT(A)
C
C***ROUTINES CALLED  ZABS
C***END PROLOGUE  ZSQRT
      DOUBLE PRECISION AR, AI, BR, BI, ZM, DTHETA, DPI, DRT
      DOUBLE PRECISION ZABS
      DATA DRT , DPI / 7.071067811865475244008443621D-1,
     1                 3.141592653589793238462643383D+0/
      ZM = ZABS(AR,AI)
      ZM = DSQRT(ZM)
      IF (AR.EQ.0.0D+0) GO TO 10
      IF (AI.EQ.0.0D+0) GO TO 20
      DTHETA = DATAN(AI/AR)
      IF (DTHETA.LE.0.0D+0) GO TO 40
      IF (AR.LT.0.0D+0) DTHETA = DTHETA - DPI
      GO TO 50
   10 IF (AI.GT.0.0D+0) GO TO 60
      IF (AI.LT.0.0D+0) GO TO 70
      BR = 0.0D+0
      BI = 0.0D+0
      RETURN
   20 IF (AR.GT.0.0D+0) GO TO 30
      BR = 0.0D+0
      BI = DSQRT(DABS(AR))
      RETURN
   30 BR = DSQRT(AR)
      BI = 0.0D+0
      RETURN
   40 IF (AR.LT.0.0D+0) DTHETA = DTHETA + DPI
   50 DTHETA = DTHETA*0.5D+0
      BR = ZM*DCOS(DTHETA)
      BI = ZM*DSIN(DTHETA)
      RETURN
   60 BR = ZM*DRT
      BI = ZM*DRT
      RETURN
   70 BR = ZM*DRT
      BI = -ZM*DRT
      RETURN
      END
      SUBROUTINE ZEXP(AR, AI, BR, BI)
C***BEGIN PROLOGUE  ZEXP
C
C     DOUBLE PRECISION COMPLEX EXPONENTIAL FUNCTION B=EXP(A)
C
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  ZEXP
      DOUBLE PRECISION AR, AI, BR, BI, ZM, CA, CB
      ZM = DEXP(AR)
      CA = ZM*DCOS(AI)
      CB = ZM*DSIN(AI)
      BR = CA
      BI = CB
      RETURN
      END
      SUBROUTINE ZLOG(AR, AI, BR, BI, IERR)
C***BEGIN PROLOGUE  ZLOG
C
C     DOUBLE PRECISION COMPLEX LOGARITHM B=CLOG(A)
C     IERR=0,NORMAL RETURN      IERR=1, Z=CMPLX(0.0,0.0)
C
C***ROUTINES CALLED  ZABS
C***END PROLOGUE  ZLOG
      DOUBLE PRECISION AR, AI, BR, BI, ZM, DTHETA, DPI, DHPI
      DOUBLE PRECISION ZABS
      DATA DPI , DHPI  / 3.141592653589793238462643383D+0,
     1                   1.570796326794896619231321696D+0/
C
      IERR=0
      IF (AR.EQ.0.0D+0) GO TO 10
      IF (AI.EQ.0.0D+0) GO TO 20
      DTHETA = DATAN(AI/AR)
      IF (DTHETA.LE.0.0D+0) GO TO 40
      IF (AR.LT.0.0D+0) DTHETA = DTHETA - DPI
      GO TO 50
   10 IF (AI.EQ.0.0D+0) GO TO 60
      BI = DHPI
      BR = DLOG(DABS(AI))
      IF (AI.LT.0.0D+0) BI = -BI
      RETURN
   20 IF (AR.GT.0.0D+0) GO TO 30
      BR = DLOG(DABS(AR))
      BI = DPI
      RETURN
   30 BR = DLOG(AR)
      BI = 0.0D+0
      RETURN
   40 IF (AR.LT.0.0D+0) DTHETA = DTHETA + DPI
   50 ZM = ZABS(AR,AI)
      BR = DLOG(ZM)
      BI = DTHETA
      RETURN
   60 CONTINUE
      IERR=1
      RETURN
      END
      DOUBLE PRECISION FUNCTION ZABS(ZR, ZI)
C***BEGIN PROLOGUE  ZABS
C
C     ZABS COMPUTES THE ABSOLUTE VALUE OR MAGNITUDE OF A DOUBLE
C     PRECISION COMPLEX VARIABLE CMPLX(ZR,ZI)
C
C***ROUTINES CALLED  (NONE)
C***END PROLOGUE  ZABS
      DOUBLE PRECISION ZR, ZI, U, V, Q, S
      U = DABS(ZR)
      V = DABS(ZI)
      S = U + V
C-----------------------------------------------------------------------
C     S*1.0D0 MAKES AN UNNORMALIZED UNDERFLOW ON CDC MACHINES INTO A
C     TRUE FLOATING ZERO
C-----------------------------------------------------------------------
      S = S*1.0D+0
      IF (S.EQ.0.0D+0) GO TO 20
      IF (U.GT.V) GO TO 10
      Q = U/V
      ZABS = V*DSQRT(1.D+0+Q*Q)
      RETURN
   10 Q = V/U
      ZABS = U*DSQRT(1.D+0+Q*Q)
      RETURN
   20 ZABS = 0.0D+0
      RETURN
      END
